import "Equendil.UI.WindowManager";
import "Equendil.UI.Resources";
import "Equendil.UI.ScrollBar";
import "Equendil.UI.VirtualListBox";
import "Equendil.UI.Resizer";
import "Equendil.Utils.Maths";
import "Equendil.Utils.Zlib";

-- controls displayed by the LIs listbox
local ItemItem = class( Turbine.UI.Control );
function ItemItem:Constructor( item )
	Turbine.UI.Control.Constructor( self );

	self.item = item;
	self.selected = false;
	
	self:SetSize( 360, 18 );
	self:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend );

	-- name, author, vesion labels ...
	self.labelName = Turbine.UI.Label();
	self.labelName:SetParent( self );
	self.labelName:SetParent( self );
	self.labelName:SetSize( 360, 18 );
	self.labelName:SetLeft( 5 );
	self.labelName:SetFont( Turbine.UI.Lotro.Font.Verdana14 );
	self.labelName:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	self.labelName:SetForeColor( Equendil.UI.Color.TurbineWhite );
	self.labelName:SetMouseVisible( false );

	self:_Update();
end

function ItemItem:_Update()
	self.labelName:SetText( self.item:GetName() );
	self:_SetBackColor();
	self:_SetForeColor();
end

function ItemItem:_SetForeColor()
	local color = Turbine.UI.Color( 1, 1, 1, 1 );
	self.labelName:SetForeColor( color );
end

function ItemItem:_SetBackColor()
	if self.selected then
		self:SetBackColor( Turbine.UI.Color( 0.2, 0.9, 0.9, 1 ) );
	else
		self:SetBackColor( Turbine.UI.Color( 0, 0, 0, 0 ) );
	end
end

-- VirtualListBox can call this to set the selection status
function ItemItem:SetSelected( flag )
	self.selected = flag;
	self:_SetBackColor();
end


local Item = class();
function Item:Constructor( itemData, name )
	Turbine.UI.Control.Constructor( self );
	self.name = name;
	self.itemData = itemData;
end

function Item:GetName()
	return self.name;
end

function Item:GetData()
	return self.itemData;
end



-- Main ChatTrackerlication class
ChatTracker = class( Turbine.UI.Lotro.Window );
function ChatTracker:Constructor()
	Turbine.UI.Lotro.Window.Constructor( self );

	-- Variables
	self.items = {}; -- LIs
	self.itemItems = {}; -- LIs listbox elements, indexed by LIs

	self.ImportRequested = nil; -- event 
	
	-- UI elements
	-- setup a window and center it
    self:SetSize( 457, 285 );
	self:SetPosition( 
		( Turbine.UI.Display:GetWidth() - self:GetWidth() ) / 2, 
		( Turbine.UI.Display:GetHeight() - self:GetHeight() ) /2 
	);	
	self:SetParent( nil );
	-- self:SetBackground( 0x41108DD9 );
	Equendil.UI.WindowManager:ManageWindow( self ); -- makes this window managed for HUD toggle / ESC
	
	self.background = Equendil.UI.DecoratedContainer( Equendil.UI.Decoration.Leatherish2 );
	self.background:SetParent( self );
	self.background:SetSize( 457, 285 );
	self.background:SetPosition( 0, 00 );
	
	-- Container for all controls related to LIs
	self.labelLIsHeaderName = Equendil.UI.CommonControl.LabelListHeading( L["Legendary items seen in the chat:"] );
	self.labelLIsHeaderName:SetParent( self.background );
	self.labelLIsHeaderName:SetSize( 377, 20 );
	self.labelLIsHeaderName:SetPosition( 40, 25 );
	self.labelLIsHeaderName:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	
	-- borders around the LIs listbox
	-- self.outlineLIs = Equendil.UI.DecoratedContainer( Equendil.UI.Decoration.RoundSilverBlue );
	-- self.outlineLIs:SetParent( self.background );
	-- self.outlineLIs:SetSize( 4, 220 );
	-- self.outlineLIs:SetPosition( , 0 );

	-- scrollbar for listbox
	self.scrollbarLIs = Equendil.UI.ScrollBar( Turbine.UI.Orientation.Vertical );
	self.scrollbarLIs:SetParent( self.background );
	self.scrollbarLIs:SetSize( 10, 180 );
	self.scrollbarLIs:SetPosition( 410, 50 );

	-- LIs listbox
	local getLIItem = function( n, selection )
		local LIItem = self:_GetNthLIItem(n);
		LIItem:_Update();
		LIItem:SetSelected( selection );
		return LIItem;
	end
	self.listboxLIs = Equendil.UI.VirtualListBox( getLIItem, 18 );
	self.listboxLIs:SetParent( self.background );
	self.listboxLIs:SetSize( 370, 160 );
	self.listboxLIs:SetPosition( 40, 50 );
	-- self.listboxLIs:SetItemHeight( 18 );
	self.listboxLIs:SetVerticalScrollBar( self.scrollbarLIs );
	self.listboxLIs:SetSelectionMode( Equendil.UI.SelectionMode.SingleSelection );
	self.listboxLIs:SetBackColor( Turbine.UI.Color( 0.1, 1, 1, 1 ) );
	self.listboxLIs:SetBackColorBlendMode( Turbine.UI.BlendMode.Overlay );

	-- buttons for actions on LIs
	self.buttonImport = Turbine.UI.Lotro.Button();
	self.buttonImport:SetParent( self.background );
	self.buttonImport:SetSize( 120, 20 );
	self.buttonImport:SetPosition( 168, 260 );
	self.buttonImport:SetText( L["Import"] );
	self.buttonImport:SetEnabled( true );
	
	self.errorLabel = Turbine.UI.Label();
	self.errorLabel:SetParent( self.background );
	self.errorLabel:SetFont( Turbine.UI.Lotro.Font.Verdana14 );
	self.errorLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	self.errorLabel:SetForeColor( Turbine.UI.Color( 1, 0.3, 0, 0 ) );
	-- self.errorLabel:SetFontStyle( Turbine.UI.FontStyle.Outline );
	self.errorLabel:SetMouseVisible( false );
	self.errorLabel:SetPosition( 50, 210 );
	self.errorLabel:SetSize( 360, 40 );
	
	-- action on LIs 
	self.listboxLIs.SelectionChanged = function() self:_LIsSelectionChanged() end
	self.buttonImport.Click = function( sender, args ) self:ImportSelection() end;
	
	self:RefreshItems();

	-- Install hook to spy the chat
	local chatHook = function (sender, args)
		local LIData, name = string.match( args.Message, "<ExamineIA:IAInfo:(.-)>(%b[])<\\ExamineIA>" ); 
		if LIData ~= nil then
			local LI = Item( LIData, string.sub( name, 2, #name - 1 ), true );
			self:_AddItem( LI );
		end
	end

	if type(Turbine.Chat.Received) == "table" then
		table.insert( Turbine.Chat.Received, chatHook );
	else
		local existingHook = Turbine.Chat.Received;
		Turbine.Chat.Received = { chatHook, existingHook };
	end
end


function ChatTracker:Add( LIData )
	if LIData ~= nil then
		local LI = Item( LIData, "Fake", true );
		self:_AddItem( LI );
	end
end

function ChatTracker:SetStatus( text )
	self.errorLabel:SetText( text );
end

function ChatTracker:_GetNthLIItem( n )
	return self.itemItems[self.items[n]];
end

function ChatTracker:RefreshItems()
	-- refresh the list of LIs
	local itemCount = #self.items;
		
	self.listboxLIs:SetItemCount( itemCount );
	self.listboxLIs:Refresh();
end

function ChatTracker:_GetSelectedItem()
	local n = self.listboxLIs:GetSelectedIndex();
	if n == -1 then return nil end
	
	return self.items[n];
end

function ChatTracker:ImportSelection()
	local item = self:_GetSelectedItem();
	if self.ImportRequested ~= nil then
		self.ImportRequested( item:GetData(), item:GetName() ); 
	end
end

-- handle selection of a LI
function ChatTracker:_LIsSelectionChanged()
	local item = self:_GetSelectedItem();
	
	self.buttonImport:SetEnabled( item ~= nil );
	if item == nil then
		return;
	end
	
	-- self.textboxDetail:SetText( item:Decode() );
end

function ChatTracker:_AddItem( item, isRaw )
	self.itemItems[item] = ItemItem( item );
	table.insert( self.items, item );
	
	if #self.items > 10 then
		self.itemItems[self.items[1]] = nil;
		table.remove( self.items, 1 );
	end
	
	self:RefreshItems();

	return item;
end

