-- 'Legacy' class to manipulate legacies and full list of all legacies for each class

-- TODO: RoI, check all adjustable legacies, need more mod values ...

import "Equendil.Utils.WeakEvent";
import "Equendil.LIP.LegacyData";


-- Legacy class
-- TODO: callbacks may be called when values remain identical despite level/age modification, would be better not to
Legacy = class( Turbine.Object )
function Legacy:Constructor( data, item )
	Turbine.Object.Constructor( self );
	
	self.legendaryItem = item;
	self.level = item:GetLevelToUse();
	self.age = item:GetAge();
	self.crystals = item:GetCrystals();
	
	self.name = data.name;
	self.type = data.type;
	self.modifierType = data.modType;
	self.modifiers = data.mod;
	self.costs = data.cost;
	self.maxRank = data.maxRank;
	self.fixRank = data.fixRank;
	self.IDs = data.IDs;
	self.ID = data.ID;
	
	self.tier = 1; -- default to tier 1
	self.rank = 1; -- default to rank 1
	
	-- items hold a list of all possible legacies, 'active' ones represent the user's choice
	self.active = false; 
	
	-- Events. No parameters
	self.Changed = Equendil.Utils.WeakEvent(); -- fired off for any change in the legacy
	
--	self.legendaryItem.Changed:Add( self.actionLegendaryItemChanged );
end

function Legacy:GetID( tier )
	if self:IsDefault() then
		if self.type == Type.DPS or self.type == Type.FuryBonusContributionRating then
			return nil;
		end

		if self.IDs == nil then
			return nil;
		end
	
		if self.type == Type.IncomingHealingRating then
			return IncomingHealingRatingID( self.level, self.age, self.crystals, self.IDs );
		else
			return self.IDs[self.age];
		end
	else
		if self.IDs == nil then 
			return nil;
		end
	
		return self.IDs[tier or self.tier];
	end
end

function Legacy:SetInternalRank( internalRank, fSetIfAdjusts )
	local rank = 1;
	if type(self.modifiers) == "table" or
		self.type == Type.IncomingHealingRating or
		self.type == Type.FuryBonusContributionRating
	then
		-- For some reasons, some legacies with fewer than 9 ranks do not have regular rank values
		if self.fixRank == 1 then
			ranks = {[47]=1,[54]=2,[61]=3,[70]=4,[78]=5};
			rank = ranks[internalRank] or 1;
		elseif self.fixRank == 2 then
			rank = (internalRank - 46) / 2;
		elseif self.fixRank == 3 then
			rank = (internalRank - 46) / ( 10 / (self.maxRank or 9) );
		else
			rank = (internalRank - 46) / ( 9 / ( self.maxRank or 9 ) );
		end
	elseif fSetIfAdjusts then
		rank = internalRank - 46 - RatingRank( self.level, self.age, self.crystals, 1 ) + 1;
	end
	rank = math.ceil( rank );
	
	self:SetRank( rank );
end

function Legacy:GetInternalRank()
	-- Default Rating ?
	if 	self.type == Type.TacticalDamageRating or
		self.type == Type.TacticalHealingRating or
		self.type == Type.ShieldUse or 
		self.type == Type.DPS
	then
		return DefaultRatingRank( self.level, self.age, self.crystals, self.rank ) + 46;
	end

	-- incoming healing rating
	if self.type == Type.IncomingHealingRating then
		return 46 + self.rank;
	end

	-- For some reasons, some legacies with fewer than 9 ranks do not have regular rank values
	if self.fixRank == 1 then
		ranks = {47,54,61,70,78};
		return ranks[self.rank];
	elseif self.fixRank == 2 then
		return 46 + (self.rank * 2);
	elseif self.fixRank == 3 then 
		return 46 + math.floor(self.rank * ( 10 / (self.maxRank or 9) ) );
	end
	
	-- Regular fixed ranks legacy ?
	if type(self.modifiers) == "table" then
		return math.floor( 46 + (self.rank * ( 9 / (self.maxRank or 9) ) ) );
	end
	
	-- Rating legacy
	return RatingRank( self.level, self.age, self.crystals, self.rank ) + 46;
end

function Legacy:GetCrystals() return self.crystals end
function Legacy:SetCrystals( crystals )
	if crystals == self.crystals then return end
	self.crystals = crystals;
	if type( self.costs ) == "function" or type( self.modifiers ) == "function" or type( self.maxRank ) == "function" then
		if type( self.maxRank ) == "function" then
			self.rank = math.min( self.rank, self.maxRank( self.level, self.age ) );
		end
		-- not entirely accurate as nothing might have actually changed, but this is a good bet if values are computed
		self.Changed(); 
	end

end

function Legacy:GetLevel() return self.level end
function Legacy:SetLevel( level ) 
	if level == self.level then return end
	self.level = level;	
	if type( self.costs ) == "function" or type( self.modifiers ) == "function" or type( self.maxRank ) == "function" then
		if type( self.maxRank ) == "function" then
			self.rank = math.min( self.rank, self.maxRank( self.level, self.age ) );
		end
		-- not entirely accurate as nothing might have actually changed, but this is a good bet if values are computed
		self.Changed(); 
	end
end

function Legacy:GetAge() return self.age end
function Legacy:SetAge( age ) 
	if age == self.age then return end
	self.age = age;
	if type( self.costs ) == "function" or type( self.modifiers ) == "function" or type( self.maxRank ) == "function" then
		if type( self.maxRank ) == "function" then
			self.rank = math.min( self.rank, self.maxRank( self.level, self.age ) );
		end
		-- not entirely accurate as nothing might have actually changed, but this is a good bet if value are computed
		self.Changed(); 
	end
end


-- Straight forward accessors & methods
function Legacy:GetName() return self.name end
function Legacy:GetType() return self.type end
function Legacy:GetRank() return self.rank end
function Legacy:GetTier() return self.tier end
function Legacy:GetMaxRank() return self:GetMaxRankLA( self.level, self.age ); end
--function Legacy:GetModifierOf( rank ) return self.modifiers[rank] end
--function Legacy:GetCurrentModifier() return self.modifiers[self.rank] end
function Legacy:GetCostOf( tier, rank )	return self:GetCostOfLA( self.level, self.age, tier, rank ); end
function Legacy:GetModifierStringOf(rank) return self:GetModifierValueStringOf(rank) .. self.name; end
function Legacy:GetCurrentModifierValueString()	return self:GetModifierValueStringOf( self.rank ); end
function Legacy:GetCurrentModifierString() return self:GetModifierStringOf( self.rank ) end
function Legacy:GetTotalCurrentCost() return self:GetTotalCostOf( self.tier, self.rank ); end
function Legacy:IsActive() return self.active end

function Legacy:SetActive( status ) self.active = status; self.Changed(); end

function Legacy:GetMaxTier()
	if self:IsDefault() then
		return 1;
	else
		return 6;
	end
end

-- returns how many ranks a legacy has at a given level/age
function Legacy:GetMaxRankLA( level, age )
	if type( self.maxRank ) == "number" then
		return self.maxRank;
	elseif type( self.maxRank ) == "function" then
		return self.maxRank( level, age );
	else
		return 9; -- legacies have 9 ranks by default
	end
end

-- returns the cost of a legacy at a given level, age, tier & rank
function Legacy:GetCostOfLA( level, age, tier, rank )
	if rank == 1 then return 0 end; -- rank 1 is always free
	
	if type(self.costs) == "table" then
		return self.costs[tier][rank-1];
	elseif type(self.costs) == "function" then
		return self.costs( level, age, tier, rank );
	end
	
	return 0; -- default, shouldn't happen
end

-- returns the modifier value of a legacy at a given level, age & rank
function Legacy:GetModifierValueOfLA( level, age, crystals, rank )
	if type( self.modifiers ) == "table" then
		return self.modifiers[rank];
	elseif type( self.modifiers ) == "function" then
		return self.modifiers( level, age, crystals, rank );
	end
	
	return 0; -- default, shouldn't happen
end



-- set the rank of a legacy
-- not testing bounds
function Legacy:SetRank( rank ) 
	if self.rank ~= rank then
		self.rank = math.min( self:GetMaxRank(), rank );
		self.rank = math.max( 1, self.rank );
		self.Changed();
		-- self.CostChanged();
	end
end

-- set the tier of a legacy
-- not testing bounds
function Legacy:SetTier( tier ) 
	if self.tier ~= tier then
		self.tier = tier;
		self.Changed();
		-- self.CostChanged();
	end
end

-- Get the total cost of a legacy of tier <tier> at rank <rank>
function Legacy:GetTotalCostOf( tier, rank )
	local total = 0;
	for i = 2, rank do
		total = total + self:GetCostOf( tier, i );
	end
	return total;
end

-- Returns whether a legacy is the default legacy for a legendary item (DPS, Tactical Damage Rating etc)
function Legacy:IsDefault()	
	return not ( self.type == Type.Minor or self.type == Type.Major or self.type == Type.Stat ); 
end

local ModFormat = {
	[Equendil.LIP.Modifier.Number] = "%+g ",
	[Equendil.LIP.Modifier.Percent] = "%+g%% ",
	[Equendil.LIP.Modifier.Cooldown] = "%+g ",
	[Equendil.LIP.Modifier.Resist] = "%+d ",
	[Equendil.LIP.Modifier.Mitigation] = "%+d ",
	[Equendil.LIP.Modifier.Rating] = "%+g ",
	[Equendil.LIP.Modifier.DPS] = "%g ",
	[Equendil.LIP.Modifier.Duration] = "+%g ",
	[Equendil.LIP.Modifier.Range] = "%+g "
}

function Legacy:GetModifierValueStringOf( rank )
	local modifier = self:GetModifierValueOfLA( self.level, self.age, self.crystals, rank );

	-- cooldown reducations are entered as positive values for convenience, so reversing that here
	if self.modifierType == Equendil.LIP.Modifier.Cooldown then
		modifier = -modifier;
	end
	
	if ( modifier == -1 ) and (
		( self.type == Equendil.LIP.Type.DPS ) or  
		( self.type == Equendil.LIP.Type.TacticalDamageRating ) or  
		( self.type == Equendil.LIP.Type.TacticalHealingRating ) or  
		( self.type == Equendil.LIP.Type.IncomingHealingRating ) or  
		( self.type == Equendil.LIP.Type.ShieldUse )
	) then
		return "? ";
	end
	
	
	return string.format( ModFormat[self.modifierType], modifier );
end
