-- re-implementation of Turbine's lotro buttons that don't accept SetMouseVisible( false ) ...
import "Turbine.UI";

local leftCapRes = {0x410001AB,0x410001A5,0x410001AB};
local middleRes = {0x410001B2,0x410001AC,0x410001B2};
local rightCapRes = {0x410001B9,0x410001B3,0x410001B9};

-- A basic custom three states button (normal, highlighted on mouse over, depressed when clicked)
LotroButton = class( Turbine.UI.Control );
function LotroButton:Constructor( width, height ) -- Icons: table of 3 elements, normal icon, highlighted icon, depressed icon
	Turbine.UI.Control.Constructor( self );	

	-- events
	self.Clicked = Equendil.Utils.Event(); -- Clicked(button) where 'button' is one of Turbine.UI.MouseButton values
	self.Hovering = Equendil.Utils.Event(); -- Hovering( flag ) is called whenever the point enters (true) or leaves (false) the button

	self.isPointerInside = false;

	self.leftCap = Turbine.UI.Control();
	self.leftCap:SetParent( self );
	self.leftCap:SetSize( 20, 20 );
	self.leftCap:SetMouseVisible( false );
	self.leftCap:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	
	self.middle = Turbine.UI.Control();
	self.middle:SetParent( self );
	self.middle:SetSize( 1, 20 );
	self.middle:SetLeft( 20, 0 );
	self.middle:SetMouseVisible( false );
	self.middle:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	
	self.rightCap = Turbine.UI.Control();
	self.rightCap:SetParent( self );
	self.rightCap:SetSize( 20, 20 );
	self.rightCap:SetMouseVisible( false );
	self.rightCap:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	
	self:_SetRes( 1 );

	local label = Turbine.UI.Label();
	label:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 );
	label:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	label:SetForeColor( Equendil.UI.Color.TurbineYellow );
	label:SetFontStyle( Turbine.UI.FontStyle.Outline );
	label:SetMouseVisible( false );
	label:SetParent( self );
	self.label = label;
	
	self.MouseEnter = function( sender, args ) 
		self:_SetRes( 2 );
		self.isPointerInside = true;
		self.Hovering( true );
	end;
	
	self.MouseLeave = function( sender, args ) 
		self:_SetRes( 1 );
		self.isPointerInside = false;
		self.Hovering( false );
	end;
	
	self.MouseDown = function( sender, args ) 
		self:_SetRes( 3 );
	end;
	
	self.MouseUp = function( sender, args ) 
		self:_SetRes( 2 );
		if self.isPointerInside then
			self.Clicked( args.Button );
		end
	end
	
	self:SetSize( width, height );
end

function LotroButton:_SetRes( n )
	self.leftCap:SetBackground( leftCapRes[n] );
	self.middle:SetBackground( middleRes[n] );
	self.rightCap:SetBackground( rightCapRes[n] );
end

function LotroButton:SetText( text )
	self.label:SetText( text );
end

function LotroButton:_DoLayout()
	self.middle:SetSize( self:GetWidth() - 40 );
	self.rightCap:SetLeft( self:GetWidth() - 20 );
	self.label:SetSize( self:GetSize() );
end

function LotroButton:SetSize( width, height )
	Turbine.UI.Control.SetSize( self, width, height );
	self:_DoLayout();
end

function LotroButton:SetWidth( width )
	self:SetSize( width, self:GetHeigth() );
end

function LotroButton:SetHeight( height )
	self:SetSize( self:GetWidth(), height );
end